@@ -1,11 +1,12 @@
 ;;; tramp-util.el --- Misc utility functions to use with Tramp
 
 ;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-;;   2008 Free Software Foundation, Inc.
+;;   2008, 2010 Free Software Foundation, Inc.
 
 ;; Author: Kai Großjohann <kai.grossjohann@gmx.net>
 ;;         Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, extensions, processes
+;; Package: tramp
 
 ;; This file is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -28,7 +29,7 @@
 ;;; Code:
 
 (require 'tramp)
-(require 'tramp-compat)
+(require 'tramp-sh)
 
 ;; Utility functions.
 
@@ -63,29 +64,24 @@
 
   (defadvice executable-find
     (around tramp-advice-executable-find activate)
-    "Invoke `tramp-handle-executable-find' for Tramp files."
+    "Invoke `tramp-sh-handle-executable-find' for Tramp files."
     ;; The check for `load-in-progress' is necessary because during
     ;; loading the usual trick setting `default-directory' to a local
     ;; filename seems not to work.  On the other hand, it might
     ;; prevent `executable-find' checks in packages which expect the
     ;; executables on remote hosts.  Needs a better solution.
-    (if (not load-in-progress)
-	(let ((fnh (tramp-find-foreign-file-name-handler default-directory)))
-	  (cond ((eq fnh 'tramp-sh-file-name-handler)
-		 (setq ad-return-value
-		       (apply 'tramp-handle-executable-find (ad-get-args 0))))
-		((eq fnh 'tramp-fish-file-name-handler)
-		 (setq ad-return-value
-		       (apply 'tramp-fish-handle-executable-find
-			      (ad-get-args 0))))
-		(t ad-do-it)))
+    (if (and (not load-in-progress)
+	     (eq (tramp-find-foreign-file-name-handler default-directory)
+		 'tramp-sh-file-name-handler))
+	(setq ad-return-value
+	      (apply 'tramp-sh-handle-executable-find (ad-get-args 0)))
       ad-do-it))
   (add-hook 'tramp-util-unload-hook
 	    '(lambda () (ad-unadvise 'executable-find)))
 
   (defadvice start-process
     (around tramp-advice-start-process activate)
-    "Invoke `tramp-handle-start-process' for Tramp files."
+    "Invoke `tramp-sh-handle-start-file-process' for Tramp files."
     (if (eq (tramp-find-foreign-file-name-handler default-directory)
 	    'tramp-sh-file-name-handler)
 	(setq ad-return-value (apply 'start-file-process (ad-get-args 0)))
@@ -95,7 +91,7 @@
 
   (defadvice start-process-shell-command
     (around tramp-advice-start-process-shell-command activate)
-    "Invoke `tramp-handle-start-process-shell-command' for Tramp files."
+    "Invoke `tramp-sh-handle-shell-command' for Tramp files."
     (if (eq (tramp-find-foreign-file-name-handler default-directory)
 	    'tramp-sh-file-name-handler)
 	(with-parsed-tramp-file-name default-directory nil
@@ -111,10 +107,9 @@
 
   (defadvice call-process
     (around tramp-advice-process-file activate)
-    "Invoke `tramp-handle-process-file' for Tramp files."
-    (if (memq (tramp-find-foreign-file-name-handler default-directory)
-	      '(tramp-sh-file-name-handler
-		tramp-fish-file-name-handler))
+    "Invoke `tramp-sh-handle-process-file' for Tramp files."
+    (if (eq (tramp-find-foreign-file-name-handler default-directory)
+	    'tramp-sh-file-name-handler)
 	(setq ad-return-value (apply 'process-file (ad-get-args 0)))
       ad-do-it))
   (add-hook 'tramp-util-unload-hook
@@ -122,12 +117,11 @@
 
   (defadvice call-process-region
     (around tramp-advice-call-process-region activate)
-    "Invoke `tramp-handle-call-process-region' for Tramp files."
-    (if (memq (tramp-find-foreign-file-name-handler default-directory)
-	      '(tramp-sh-file-name-handler
-		tramp-fish-file-name-handler))
+    "Invoke `tramp-sh-handle-call-process-region' for Tramp files."
+    (if (eq (tramp-find-foreign-file-name-handler default-directory)
+	    'tramp-sh-file-name-handler)
 	(setq ad-return-value
-	      (apply 'tramp-handle-call-process-region (ad-get-args 0)))
+	      (apply 'tramp-sh-handle-call-process-region (ad-get-args 0)))
       ad-do-it))
   (add-hook 'tramp-util-unload-hook
 	    '(lambda () (ad-unadvise 'call-process-region)))
@@ -432,8 +426,7 @@ Works only for relative file names and Tramp file names."
 	      ;;-   method user host
 	      ;;-   (format "echo %s" (comint-quote-filename localname))))
 	      (tramp-send-command v (format "echo %s" localname))
-	      (setq bufstr (buffer-substring
-			    (point-min) (tramp-compat-line-end-position)))
+	      (setq bufstr (buffer-substring (point-min) (point-at-eol)))
 	      (with-current-buffer (tramp-get-buffer v)
 		(goto-char (point-min))
 		(if (string-equal localname bufstr)
@@ -463,10 +456,12 @@ Works only for relative file names and Tramp file names."
        (add-hook 'tramp-unload-hook
 		 '(lambda () (ad-unadvise 'PC-expand-many-files))))))
 
+(add-hook 'tramp-unload-hook
+	  '(lambda ()
+	     (unload-feature 'tramp-util 'force)))
 
 (provide 'tramp-util)
 
-;;; arch-tag: 500f9992-a44e-46d0-83a7-980799251808
 ;;; tramp-util.el ends here
 
 ;; Local Variables:
